{******************************************************************************}
{ Jazarsoft Virtual Desktop Component                                          }
{******************************************************************************}
{                                                                              }
{ VERSION      : 1.0                                                           }
{ AUTHOR       : James Azarja                                                  }
{ CREATED      : 10 July 2000                                                  }
{ WEBSITE      : http://www.jazarsoft.cjb.net/                                 }
{ SUPPORT      : support@jazarsoft.cjb.net                                     }
{ BUG-REPORT   : bugreport@jazarsoft.cjb.net                                   }
{ COMMENT      : comment@jazarsoft.cjb.net                                     }
{ LEGAL        : Copyright (C) 2000 Jazarsoft.                                 }
{                                                                              }
{******************************************************************************}
{ NOTE         :                                                               }
{                                                                              }
{ This code may be used and modified by anyone so long as  this header and     }
{ copyright  information remains intact.                                       }
{                                                                              }
{ The code is provided "as-is" and without warranty of any kind,               }
{ expressed, implied or otherwise, including and without limitation, any       }
{ warranty of merchantability or fitness for a  particular purpose.           }
{                                                                              }
{ In no event shall the author be liable for any special, incidental,          }
{ indirect or consequential damages whatsoever (including, without             }
{ limitation, damages for loss of profits, business interruption, loss         }
{ of information, or any other loss), whether or not advised of the            }
{ possibility of damage, and on any theory of liability, arising out of        }
{ or in connection with the use or inability to use this software.           }
{                                                                              }
{******************************************************************************}

unit VirtualDesktop;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TWindowObject = record
                    WinHandle       : HWnd;
                    WinCaption      : String;
                    ProcessID       : Integer;
                    IsVisible       : Boolean;
                    IsEnabled       : Boolean;
                    IsIconic        : Boolean;
                    WindowRect      : TRect;
                    WindowPlacement : TWindowPlacement;
                  end;
  PTWindowObject = ^TWindowObject;

  TDesktop = class(TPersistent)
  private
    WindowLst : TList;
    ForeWindow: Hwnd;
    FCount    : Integer;
    HwndList  : tStrings;
  protected
    Function GetAWindow(Index : Integer) : TWindowObject;
  public
    constructor Create;
    destructor Destroy; override;

    Procedure Refresh;
    Property  Windows[Index : Integer]: TWindowObject read GetAWindow;
    Property  Count : Integer read FCount;
  published
  end;

  TVirtualDesktop = class(TComponent)
  private
    FNumberOfDesktop : Integer;
    FVirtuaDesktop   : Array of TDesktop;
    FCurrentDesktop  : Integer;
    FExcludeWindow   : tStrings;
    FNewCreatedWindow: tStrings;
    FShowNewWindow   : Boolean;
    Procedure SetTotalDesktop(Value:Integer);
    Procedure SetCurrentDesktop(Value:Integer);
  protected
    Procedure SaveDesktop(Index:Integer);
    Procedure RestoreDesktop(Index,OldIndex:Integer);
    Function  GeTVirtualDesktop(Index:Integer):tDesktop;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    Property   VirtuaDesktop[Index : Integer]: TDesktop read GeTVirtualDesktop;
    Procedure Start;
    Procedure Stop;
    Procedure ResetAllDesktop;
    Procedure RestoreAllWindow;
  published
    property NumberOfDesktop            : Integer  Read FNumberOfDesktop Write SetTotalDesktop;
    property CurrentDesktop             : Integer  Read FCurrentDesktop  Write SetCurrentDesktop;
    property ExcludeWindow              : tStrings Read FExcludeWindow   Write FExcludeWindow;
    property ShowNewCreatedWindowOnExit : Boolean  Read FShowNewWindow   Write FShowNewWindow;
  end;

procedure Register;

implementation

Function WindowCallback(WHandle : HWnd; Var Parm : Pointer) : Boolean; stdcall;
 Var MyString : PChar;
     MyInt : Integer;
     MyWindowPtr : ^TWindowObject;
begin
    New(MyWindowPtr);

    MyWindowPtr.WinHandle := WHandle;

    MyString := Allocmem(255);
    GetWindowText(WHandle,MyString,255);
    MyWindowPtr.WinCaption := String(MyString);
    FreeMem(MyString,255);

    MyInt := 0;
    MyWindowPtr.ProcessID := GetWindowThreadProcessId(WHandle,@MyInt);

    MyWindowPtr.IsVisible := IsWindowVisible(WHandle);

    MyWindowPtr.IsEnabled := IsWindowEnabled(WHandle);

    MyWindowPtr.IsIconic := IsIconic(WHandle);

    MyWindowPtr.WindowRect := Rect(0,0,0,0);
    GetWindowRect(WHandle,MyWindowPtr.WindowRect);
 
    MyWindowPtr.WindowPlacement.length:=SizeOf(tWindowPlacement);
    GetWindowPlacement(WHandle,@MyWindowPtr.WindowPlacement);

    TList(Parm).Add(MyWindowPtr);
    Result := True; 
end;

constructor TDesktop.Create;
begin
  inherited;
  WindowLst := TList.Create;
  HwndList:=tStringList.Create;
end;

destructor TDesktop.Destroy;
 var I : Integer;
begin
  If WindowLst.Count > 0 Then
    Begin
      For I := 0 To (WindowLst.Count - 1) Do
        Dispose(PTWindowObject(WindowLst[I]));
    End;
  WindowLst.Free;
  HwndList.Free;
  inherited;
end;

procedure TDesktop.Refresh;
var Counter:Integer;
begin
  WindowLst.Clear; {Clear the list!}
  HwndList.Clear;
  EnumWindows(@WindowCallback,Longint(@WindowLst));
  FCount := WindowLst.Count;
  ForeWindow:=GetForegroundWindow;
  For Counter:=0 to FCount-1 do
   HwndList.Add(InttoHex(Windows[Counter].WinHandle,8));
end;

function TDesktop.GetAWindow(Index : Integer) : TWindowObject;
begin
  Result := PTWindowObject(WindowLst[Index])^;
end;


constructor TVirtualDesktop.Create(AOwner: TComponent);
var Counter : Integer;
begin
  FExcludeWindow:=tstringList.Create;
  FNewCreatedWindow:=tStringList.Create;
  FNumberOfDesktop:=2;
  FCurrentDesktop:=1;
  
  If Not ( csDesigning in ComponentState ) Then
  Begin
   SetLength(FVirtuaDesktop,FNumberOfDesktop);
   For Counter:=0 to FNumberOfDesktop-1 do
    Begin
     FVirtuaDesktop[Counter]:=TDesktop.Create;
    End;
  End;
  inherited;
end;

destructor TVirtualDesktop.Destroy;
var Counter : Integer;
begin
  If Not ( csDesigning in ComponentState ) Then
  Begin
   For Counter:=0 to FNumberOfDesktop-1 do
     FVirtuaDesktop[Counter].Free;
  End;
  FExcludeWindow.Free;
  FNewCreatedWindow.Free;
  inherited;
end;

Procedure TVirtualDesktop.SetTotalDesktop(Value:Integer);
var Counter:Integer;
Begin
 If Value<>FNumberOfDesktop then
 Begin
  If Value<FNumberOfDesktop then { Dec }
  Begin

   If Not ( csDesigning in ComponentState ) Then
   Begin
    SetLength(FVirtuaDesktop,Value);
    For Counter:=FNumberOfDesktop-1 downto Value do FVirtuaDesktop[Counter].Free;
    SetLength(FVirtuaDesktop,Value);
   End;

  End else
  If Value>FNumberOfDesktop then
  Begin

   If Not ( csDesigning in ComponentState ) Then
   Begin
    SetLength(FVirtuaDesktop,Value);
    For Counter:=FNumberOfDesktop to Value-1 do
    Begin
     FVirtuaDesktop[Counter]:=TDesktop.Create;
//     FVirtuaDesktop[Counter].Refresh;
    End;
   End;

  End;
  FNumberOfDesktop:=Value;
 End;
End;


Procedure TVirtualDesktop.ResetAllDesktop;
var Counter:Integer;
Begin
 For Counter:=0 to FNumberOfDesktop-1 do
 Begin
  FVirtuaDesktop[Counter].Refresh;
 End;
End;

Procedure TVirtualDesktop.SaveDesktop(Index:Integer);
Begin
 If Index<=FNumberOfDesktop then
 Begin
  FVirtuaDesktop[Index-1].Refresh;
 End;
{  For Counter:=0 to FVirtuaDesktop[Index-1].Count-1 do
  Begin

   With FVirtuaDesktop[Index-1].Windows[Counter] do
   Begin
    ShowWindow(WinHandle,SW_HIDE);
   End;

  End;
}End;

Procedure TVirtualDesktop.RestoreDesktop(Index,OldIndex:Integer);
var Counter : Integer;
    HwndIdx : Integer;
Begin
 If Index<=FNumberOfDesktop then
 Begin
  For Counter:=0 to FVirtuaDesktop[OldIndex-1].Count-1 do
  Begin

   With FVirtuaDesktop[OldIndex-1].Windows[Counter] do
   Begin
    if FExcludeWindow.IndexOf(InttoHex(WinHandle,8))<>-1 then Continue;

    HwndIdx:=FVirtuaDesktop[Index-1].HwndList.IndexOf(InttoHex(WinHandle,8));

    If HwndIdx <>-1 then { Already created before }
    Begin

     If not FVirtuaDesktop[Index-1].Windows[HwndIdx].isVisible then
     Begin
      ShowWindow(WinHandle,SW_HIDE);
     End;

    End else { new Window }
    Begin
     if isVisible then
      If FNewCreatedWindow.IndexOf(InttoHex(WinHandle,8))=-1 then
       FNewCreatedWindow.Add(InttoHex(WinHandle,8));

      ShowWindow(WinHandle,SW_HIDE);
    end;
   End;

  End;

  For Counter:=0 to FVirtuaDesktop[Index-1].Count-1 do
  Begin

   With FVirtuaDesktop[Index-1].Windows[Counter] do
   Begin
    if FExcludeWindow.IndexOf(InttoHex(WinHandle,8))<>-1 then Continue;

    If Not isVisible then
    Begin
     Continue;
    end;
    WindowPlacement.Length:=Sizeof(TWindowPlacement);
    SetWindowPlacement(WinHandle,@WindowPlacement);
   End;

  End;

  SetForegroundWindow(FVirtuaDesktop[Index-1].ForeWindow);
 End;

End;

Procedure TVirtualDesktop.SetCurrentDesktop(Value:Integer);
Begin
 If Value<>FCurrentDesktop then
 Begin
  If Value>FNumberOfDesktop then
  begin
   ShowMessage('Desktop was not defined');
   Exit;
  End;
  SaveDesktop(FCurrentDesktop);
  RestoreDesktop(Value,FCurrentDesktop);
  FCurrentDesktop:=Value;
 End;
End;

Function TVirtualDesktop.GeTVirtualDesktop(Index:Integer):tDesktop;
Begin
 If Index<=FNumberOfDesktop then
 Begin
  Result:=FVirtuaDesktop[Index-1];
 End;
End;

Procedure TVirtualDesktop.Start;
Begin
 ResetAllDesktop;
End;

Procedure TVirtualDesktop.RestoreAllWindow;
var Counter:Integer;
Begin
 For Counter:=0 to FNewCreatedWindow.Count-1 do
 Begin
 if FShowNewWindow then
  ShowWindow(StrToInt('$'+FNewCreatedWindow[Counter]),SW_NORMAL);
 End;
End;

Procedure TVirtualDesktop.Stop;
Begin
 RestoreAllWindow;
End;

procedure Register;
begin
  RegisterComponents('Jazarsoft', [TVirtualDesktop]);
end;

end.
